@@ -58,35 +58,43 @@ static char **completion_fn(const char *text, int start, int end)
 #endif
 
 	/* make sure we have a list of valid commands */
-	if (!commands) 
+	if (!commands) {
 		return NULL;
+	}
 
 	matches = SMB_MALLOC_ARRAY(char *, MAX_COMPLETIONS);
-	if (!matches) return NULL;
+	if (!matches) {
+		return NULL;
+	}
 
 	matches[count++] = SMB_STRDUP(text);
-	if (!matches[0]) return NULL;
+	if (!matches[0]) {
+		SAFE_FREE(matches);
+		return NULL;
+	}
 
-	while (commands && count < MAX_COMPLETIONS-1) 
-	{
-		if (!commands->cmd_set)
+	while (commands && count < MAX_COMPLETIONS-1) {
+		if (!commands->cmd_set) {
 			break;
+		}
 		
-		for (i=0; commands->cmd_set[i].name; i++)
-		{
+		for (i=0; commands->cmd_set[i].name; i++) {
 			if ((strncmp(text, commands->cmd_set[i].name, strlen(text)) == 0) &&
 				(( commands->cmd_set[i].returntype == RPC_RTYPE_NTSTATUS &&
                         commands->cmd_set[i].ntfn ) || 
                       ( commands->cmd_set[i].returntype == RPC_RTYPE_WERROR &&
-                        commands->cmd_set[i].wfn)))
-			{
+                        commands->cmd_set[i].wfn))) {
 				matches[count] = SMB_STRDUP(commands->cmd_set[i].name);
-				if (!matches[count]) 
+				if (!matches[count]) {
+					for (i = 0; i < count; i++) {
+						SAFE_FREE(matches[count]);
+					}
+					SAFE_FREE(matches);
 					return NULL;
+				}
 				count++;
 			}
 		}
-		
 		commands = commands->next;
 		
 	}
@@ -99,9 +107,9 @@ static char **completion_fn(const char *text, int start, int end)
 	return matches;
 }
 
-static char* next_command (char** cmdstr)
+static char *next_command (char **cmdstr)
 {
-	static pstring 		command;
+	char *command;
 	char			*p;
 	
 	if (!cmdstr || !(*cmdstr))
@@ -110,7 +118,7 @@ static char* next_command (char** cmdstr)
 	p = strchr_m(*cmdstr, ';');
 	if (p)
 		*p = '\0';
-	pstrcpy(command, *cmdstr);
+	command = SMB_STRDUP(*cmdstr);
 	if (p)
 		*cmdstr = p + 1;
 	else
@@ -455,7 +463,6 @@ extern struct cmd_set spoolss_commands[];
 extern struct cmd_set netlogon_commands[];
 extern struct cmd_set srvsvc_commands[];
 extern struct cmd_set dfs_commands[];
-extern struct cmd_set reg_commands[];
 extern struct cmd_set ds_commands[];
 extern struct cmd_set echo_commands[];
 extern struct cmd_set shutdown_commands[];
@@ -470,7 +477,6 @@ static struct cmd_set *rpcclient_command_list[] = {
 	netlogon_commands,
 	srvsvc_commands,
 	dfs_commands,
-	reg_commands,
 	echo_commands,
 	shutdown_commands,
  	test_commands,
@@ -663,13 +669,11 @@ out_free:
 	}
 */
 
-	if (argv) {
-		/* NOTE: popt allocates the whole argv, including the
-		 * strings, as a single block.  So a single free is
-		 * enough to release it -- we don't free the
-		 * individual strings.  rtfm. */
-		free(argv);
-	}
+	/* NOTE: popt allocates the whole argv, including the
+	 * strings, as a single block.  So a single free is
+	 * enough to release it -- we don't free the
+	 * individual strings.  rtfm. */
+	free(argv);
 	
 	return result;
 }
@@ -679,7 +683,6 @@ out_free:
 
  int main(int argc, char *argv[])
 {
-	BOOL 			interactive = True;
 	int 			opt;
 	static char		*cmdstr = NULL;
 	const char *server;
@@ -689,6 +692,7 @@ out_free:
 	struct in_addr 		server_ip;
 	NTSTATUS 		nt_status;
 	static int		opt_port = 0;
+	fstring new_workgroup;
 
 	/* make sure the vars that get altered (4th field) are in
 	   a fixed location or certain compilers complain */
@@ -712,9 +716,7 @@ out_free:
 
 	/* the following functions are part of the Samba debugging
 	   facilities.  See lib/debug.c */
-	setup_logging("rpcclient", interactive);
-	if (!interactive) 
-		reopen_logs();
+	setup_logging("rpcclient", True);
 	
 	/* Parse options */
 
@@ -755,11 +757,22 @@ out_free:
 	if (!init_names())
 		return 1;
 
+	/* save the workgroup...
+	
+	   FIXME!! do we need to do this for other options as well 
+	   (or maybe a generic way to keep lp_load() from overwriting 
+	   everything)?  */
+	
+	fstrcpy( new_workgroup, lp_workgroup() );
+
 	/* Load smb.conf file */
 
-	if (!lp_load(dyn_CONFIGFILE,True,False,False))
+	if (!lp_load(dyn_CONFIGFILE,True,False,False,True))
 		fprintf(stderr, "Can't load %s\n", dyn_CONFIGFILE);
 
+	if ( strlen(new_workgroup) != 0 )
+		set_global_myworkgroup( new_workgroup );
+
 	/*
 	 * Get password
 	 * from stdin if necessary
@@ -772,6 +785,11 @@ out_free:
 		}
 	}
 	
+	if ((server[0] == '/' && server[1] == '/') ||
+			(server[0] == '\\' && server[1] ==  '\\')) {
+		server += 2;
+	}
+
 	nt_status = cli_full_connection(&cli, global_myname(), server, 
 					opt_ipaddr ? &server_ip : NULL, opt_port,
 					"IPC$", "IPC",  
@@ -810,6 +828,7 @@ out_free:
  
                 while((cmd=next_command(&p)) != NULL) {
                         NTSTATUS cmd_result = process_cmd(cli, cmd);
+			SAFE_FREE(cmd);
 			result = NT_STATUS_IS_ERR(cmd_result);
                 }
 		
